function net = trainrbf(net,X,Y,options)
% TRAINRBF Trains an rbfnet given training samples
%   Input:
%       net: the net object
%       X:  the inputs  [input_dimension x number of samples]
%       Y:  the targets [output_dimension x number of samples]
%       options: Currently allowed options - minerr, maxiter, displayerr, initializev
%
%   Output:
%       Trained net
%
%   Misc:
%       Currently this class only modifies weight matrices
%       The centers and widths are not modified. This will be added 
%       after bias has been taken care of.


% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /mnt/localhd/cvsdir/MODELLING/NEWDIVA/@ahrbf/trainrbf.m,v 1.1.1.1 2006/10/06 18:20:23 brumberg Exp $

% $NoKeywords: $

% Determine what options have been set
% Currently allowed options
%   minerr, maxiter, displayerr, initializev
if ~isfield(options,'minerr'),
    minerr = 0;
else,
    minerr = options.minerr;    
end;
if ~isfield(options,'maxiter'),
    maxiter = inf;
else,
    maxiter = options.maxiter;
end;
if ~isfield(options,'displayerr'),
    bDisplayError = 0; % By default no error displayed
else,
    bDisplayError = options.displayerr;    
end;
if ~isfield(options,'initializev'),
    bInitializeV = 0; % By default no error displayed
else,
    bInitializeV = options.initializev;    
end;

% Determine the weight net.v using a pseudo-inverse calculation
if bInitializeV,
    numiter = 50;
    hproc = waitbar(0,'Initializing weights');
    net.v = zeros(size(net.v));
    for i=1:numiter,
        idx = ceil(size(X,2)*rand(1,min([size(X,2) 500])));
        [Yhat,H] = simrbf(net,X(:,idx));
        net.v = net.v+[Y(:,idx)*pinv(H)];
        waitbar(i/numiter,hproc);
    end;
    close(hproc);
    net.v = net.v/numiter;
end;

if bDisplayError,
    figure(10);hold on;
end;

% Train to get net.w
E = inf;
count = 0;
while(mean(max(E,0))>0) & count<maxiter,
    count = count+1;
    % Select a random ordering of the training set
    idx = randperm(size(X,2));
    Err = 0;
    % do an online training
    for i=1:length(idx),
        %simulate
        net = netout(net,X(:,idx(i)));
        %calculate error
        err = Y(:,idx(i))-net.out;
        % update weights
        net = updatevw(net,err);
        %        cerr = mean(abs(err./(1+Y(:,idx(i)))));
        Err = (Err*(i-1)+err.^2)/i;
    end
    % Error displayed in command window should go to zero if minerr
    % is reached
    E = sqrt(Err)-minerr
    
    if bDisplayError,
        plot(count,mean(E),'o');
        drawnow;
    end;
end;