function plot(vtm,X,AV,Ar,last)
% PLOT  Plots the shape of the vocal tract
%   This function plots the shape of the vocal tract corresponding
%   to the provided articulatory positions.
%   X is the matrix of articulatory inputs
%   AV is a vector signifying voicing. Whenever AV is non-zero, a
%   signal is drawn on the body to signify that the current utterance 
%   is being voiced.

% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /mnt/localhd/cvsdir/MODELLING/NEWDIVA/@d_opvt/plot.m,v 1.3 2006/10/27 16:55:48 brumberg Exp $

% $NoKeywords: $

% Setup globals
  global RELEASE

%  X=X(1:8,:);

  if nargin<4
    if(size(X,1)>=8)
      Ar=X;
    else
      Ar=[];
    end
    last=zeros(1,size(X,2));
  elseif(nargin<5)
    last=zeros(1,size(X,2));
  end
  % In order to speed up plotting this sin function has to be taken 
  % out of the loop
  fs = 10000;
  t = [0:1/fs:0.01]';
  y = sin(2*pi*t*500);

  % scale and shift the plot to the appropriate location and size
  AVval(:,1) = 275+50/0.01*t; 
  AVval(:,2) = 525+25*y;
  % Cycle through each input
  for i=1:size(X,2),
    [Ft,Bt,At,Af{i},Tf{i},P1{i},P2t] = doAM(X(1:7,i));
    % Draw the figure
    diva_drawman(P1{i},P2t,X(1,i),AV(i),AVval,last(i));    
    % plot area function
    drawAreaFunc(X(1:7,i),vtm);
    
    [TP,TL]=miniAr2Tac(Ar(:,i),X(:,i));    
    
    % plot palatotactile info
    %drawPalate(Ar(:,i),X(:,i));
    drawPalate(TP);
    % draw lip model
    %drawLips(Ar(:,i),X(:,i));
    drawLips(TL);
    hold off;
    
    drawnow;
  end
  
  %arm wave
%   last=[1 0 1 0];
%   for i=1:4,
%     % Draw the figure
%     diva_drawman(P1{size(X,2)},P2t,X(1,size(X,2)),AV(size(X,2)),AVval,last(i));
%     % plot area function
%     drawAreaFunc(X(1:7,size(X,2)),vtm);
%     % plot palatotactile info
%     drawPalate(Ar(:,size(X,2)),X(:,size(X,2)));
%     % draw lip model
%     drawLips(Ar(:,size(X,2)),X(:,size(X,2)));
%     hold off;
    
%     drawnow;
%   end


function drawLips(TL)%Ar,X)
  %[TP,TL]=miniAr2Tac(Ar,X);
  yoff=350;
  xlow=50;
  xhigh=270;
  xstep=3;
  %TL=9-TL;
  TL=8-TL;
  hold on;
  y=ones(1,3).*yoff;
  % Upper lip
  y(2)=yoff-TL*8;
  x=linspace(xlow,xhigh,xstep);
  xi=linspace(xlow,xhigh);
  yi=interp1(x,y,xi,'pchip');
  plot(xi,yi,'k','LineWidth',5);
  % Lower lip
  y(2)=yoff+TL*8;
  x=linspace(xlow,xhigh,xstep);
  xi=linspace(xlow,xhigh);
  yi=interp1(x,y,xi,'pchip');
  plot(xi,yi,'k','LineWidth',5);
  hold off;
  return

function drawPalate(TP)%Ar,X)
% c=1; a=a;b=b;
% C=pi*sqrt(a^2+b^2);
% L=C/n; % where n=2*pnode-1;
% ra=(c^2*a^2*b^2);
% rb=(b^2*(cos(t).^2))+(a^2*(sin(t).^2));
% r=sqrt(ra./rb);
% T(1)=0;
% T(n)=T(n-1)+L/(r(floor(T(n-1)*10000+1)));
% R=r(floor(T*10000+1));
% x=R.*cos(T);
% y=R.*sin(T);
%  figure,hold on
  %[TP,TL]=miniAr2Tac(Ar,X);
  
  xoff=130;
  
  y_min=650;
  x_min=40;
  amp=50;
  amp_step=12;
  hold on
  yoff=250;
  b=[3.6 3.9 3.2 3.0 2.8 2.6 2.4 2.2 2.0];
  a=[3.7 3.9 3.3 2.8 2.3 1.8 1.3 0.8 0.6];
  c=1;
  cc=2;
  %t=0:0.0001:pi;
  t=pi/2;
  pnode=[9 8 7 5 4];
  j=1;
  theta=linspace(pi/2,3*pi/2,200);
  tlabel={'l1','l2','l3','l4','l5'};
  for i=1:1:9
    xt=[];
    yt=[];
    if(~mod(i,2)),
      xt=a(i)*sin(theta);
      yt=b(i)*cos(theta);
      xt=(xt+1)*35+xoff;
      yt=yt*amp+250;

      cstr='k';
    else
      n=pnode(j)*4;
      cc=2;
      t=pi/2;
      h=((a(i)-b(i))^2)/((a(i)+b(i))^2);
      C=pi*(a(i)+b(i))*(1+((3*h)/(10+sqrt(4-3*h))));
      C=C/4;
      L=C/n; % where n=2*pnode-1;
      if j<=3
        L=L+(.1*L);
      elseif j==5
        L=L-(.05*L);
      end
      ra=(c^2*a(i)^2*b(i)^2);
      rb=(b(i)^2*(cos(t)^2))+(a(i)^2*(sin(t)^2));
      r=sqrt(ra/rb);
      T=[];
      R=[];
      T(1)=t;
      R(1)=r;
      for k=2:n
        t=t+L/r;
        rb=((b(i)^2)*(cos(t)^2))+((a(i)^2)*(sin(t)^2));
        r=sqrt(ra/rb);
        if mod(k-1,4)==0
          T(cc)=t;
          R(cc)=r;
          cc=cc+1;
        end
      end
      xt=R.*cos(T);
      yt=-R.*sin(T);
      cstr='.k';      
      order=length(xt):-1:1;
      xt=xt(order);
      yt=yt(order);
      order=length(xt)-1:-1:1;

      xt=([xt -xt(order)]+1)*35+xoff;
      yt=[yt yt(order)]*amp+250;

      
      tpoints.(char(tlabel(j))).x=xt;
      tpoints.(char(tlabel(j))).y=yt;
      j=j+1;

    end
    plot(xt,yt,cstr);
  end
  j=1;
  TP=TP+1;
  for i=1:5
    idx=6-i;
    plot(tpoints.(char(tlabel(i))).x(TP(i):-1:1), ...
         tpoints.(char(tlabel(i))).y(TP(i):-1:1),'*r')
    plot(tpoints.(char(tlabel(i))).x(end+1-(TP(i):-1:1)), ...
         tpoints.(char(tlabel(i))).y(end+1-(TP(i):-1:1)),'*r')
  end
  hold off
  return
  
function [TP,TL]=miniAr2Tac(Ar,X)
  if ~isempty(Ar)
    sizeAr2=size(Ar,2);
  else
    sizeAr2=1;
  end
  
  Maeda2Palate=[ 4.9750    0.3910   -1.5280    0.3720    1.7100 0; ...
                 3.8020    0.3940   -1.5770    0.6160    1.4490 0; ...
                 2.1380    0.1940   -1.4440    0.6380    0.7930 0; ...
                 0.3780    0.2830   -1.2290    0.6260    0.3220 0; ...
                 -0.4840   0.3880   -0.8910    0.7030    0.3910 0; ...
                 4.9190    0.2710    0.0280   -0.0040   -0.0010 -1.8114];
  MaedaParam=X;
  %size(Ar,2)
  T = max(min(Maeda2Palate*[ones(1,sizeAr2);MaedaParam(1:5,: )], ...
              repmat([9 8 7 5 4 9]',1,sizeAr2)),0);
  %S=[Ar;T];
  %T=S(9:end,:);
  TP=T(1:end-1);
  TP=floor(TP);
  TL=T(end);
  return
  
function drawAreaFunc(X,vtm)
  tdata.X=X;
  tdata=getAreaFunc(vtm,tdata);
  y_min=650;
  x_min=40;
  bar_step=15;
  y_amp=20;
  hold on;
  for i=1:length(tdata.Af)
    line([x_min+i*bar_step,x_min+i*bar_step+bar_step],[y_min- ...
                        tdata.Af(i)*y_amp,y_min-tdata.Af(i)*y_amp], ...
         'color','k');
    if(i>1)
      line([x_min+i*bar_step,x_min+i*bar_step],[y_min- tdata.Af(i)* ...
                          y_amp,y_min-tdata.Af(i-1)* y_amp],'color','k');
    end
  end
  [mina mini]=min(tdata.Af);
  tot=length(tdata.Af);
  set(gca,'xtick',[],'xticklabel',[],'ytick',[],'yticklabel',[]);
  xlabel(sprintf('Min Area: %2.1f cm^2 Tube %d of %d',mina,mini,tot));
  return
  
