function sig = diva_synth2(data)
% DIVA_SYNTH2   Synthesizes the sound
%   This core function relies on a mex file to provide the output signal
%   Input:
%   data: The structure data must have the following fields
%       Af,Ag0,AgP,F0, duration, dx and optionally fs

% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /mnt/localhd/cvsdir/MODELLING/NEWDIVA/@d_opvt/private/diva_synth2.m,v 1.7 2006/10/25 21:31:49 brumberg Exp $

% $NoKeywords: $

% Setup globals
global RELEASE

global diva_nf;
global diva_af;
global diva_sig;

if ~isfield(data,'Af') & isfield(data,'X'),
    data.Af = [];data.dx = [];
    for i=1:size(data.X,2),   
        [F,B,A,Af] = doAM(data.X(:,i));
        data.Af(:,i) = Af(1,:)';
        data.dx(:,i) = Af(2,:)';
    end;
end;

diva_nf=data.Af;
[data.Af,sidx,eidx]=AfRelease(data.Af);
diva_af=data.Af;

% Determine the dimensionality of the area functions
n = size(data.Af,2);

% Initialize all parameters for vtsynth - [note the transpose]
Ag0 = data.Ag0';
AgP = data.AgP';
F0  = data.F0';

TAF1 = [linspace(0,data.duration,n)' [0;2 * ones(n-1,1)] 1*ones(n,1)]';
TAF2 = data.Af;

% call vtsynth to generate the signal
if isfield(data,'fs'),
  [sig] = vtsynth(size(data.Af,1),Ag0,AgP,F0,TAF1,TAF2,data.dx, data.fs,0);
  % vtsynth can take 7 or 9 params, if 9, then 9th param follows:
  %   1: output tube with min area
  %   0: do nothing
else,
  data.fs=10000;
  [sig] = vtsynth(size(data.Af,1),Ag0,AgP,F0,TAF1,TAF2,data.dx);
end  

% In order to reduce clicks we are multiplying the signal by a voicing
% component defined by the minimum of the area function at each point

AV=interp1(linspace(1,length(sig),n),min(data.Af)>0.001,1:length(sig))';
sig = sig.*AV;
diva_sig=sig;

B=fir1(10,[1000/(data.fs/2) 3000/(data.fs/2)]);

if(length(sidx)==length(eidx))
  for n=1:length(sidx),
    %sig((sidx(n)*10):((eidx(n)-16)*10))=0;
    sig((sidx(n)*10):(eidx(n)*10))=fftfilt(B,sig((sidx(n)*10):(eidx(n)*10)));
  end
end


% Voicing only after steady state released (Not beginning of
% production due to movement from initial position)

%sig(1:(15*10))=0;

