function [data] = diva_synthX(data)
% DIVA_SYNTH2   Synthesizes the sound
%   This core function relies on a mex file to provide the output signal
%   Input:
%   data: The structure data must have the following fields
%       Af,Ag0,AgP,F0, duration, dx

% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /mnt/localhd/cvsdir/MODELLING/NEWDIVA/@d_opvt/private/diva_synthX.m,v 1.1.1.1 2006/10/06 18:20:23 brumberg Exp $

% $NoKeywords: $

% Setup globals
global RELEASE

TYPE='Tf';         % 'Tf' synthesis from transfer function; 'Af' synthesis from area function
Fs=data.fs;
sigm=inline('(1-exp(-x))./(1+exp(-x))');
if ~isfield(data,TYPE) & isfield(data,'X'),
    Nt=size(data.X,2); %data.dx=zeros(1,Nt);
    data.Tf=zeros(125,Nt); for i=1:Nt,   
        [F,B,A,Af,Tf,P1] = doAM(data.X(:,i));
        if i==1, data.Af=zeros([size(Af,2),Nt]); opt.P1=zeros([size(P1),Nt]); end 
        % computes uniform-sampling transfer function (Tf has a non-uniform sampling)
        incr=[0;0;2*(abs(diff(Tf(2:end)))<1)-1]; df=40*ones(size(Tf)); for n1=2:length(Tf), df(n1)=max(.5,min(40,df(n1-1)+incr(n1-1)*8)); end; f=cumsum(df);
        data.Tf(ceil(f/40),i)=Tf;
        data.Af(:,i) = Af(1,:)';
        opt.P1(:,:,i)=P1;
        data.dx(:,i)=Af(2,:)';
        %plot(i*ones(size(F)),F,'k.'); hold on; drawnow;
        %subplot(221); plot(P1(1,:),P1(2,:),'b',P1(3,:),P1(4,:),'r'); axis equal; subplot(222); plot(Af');
        %subplot(223); plot(i*ones(size(F)),F,'.'); hold on; subplot(224); plot(Tf); hold on; drawnow;
        %disp(round(sum(df)))
        %if ~isempty(Tf), data.Tf(:,i)=Tf(ceil((1:256)/256*length(Tf))); end; 
        %data.dx=data.dx+mean(Af(2,:));
    end;
    if 1,
        % re-interpolation of Area function
        Na=size(data.Af,1);
        Nfact=16;
        data.Af=shiftdim(max(0,min(reshape(real(exp(-j*2*pi*(-63:64)'/128*linspace(-.5,Na-.5,Na*Nfact))'*exp(j*2*pi*(0:Na-1)'*(-63:64)/128)'*data.Af/128),[Nfact,Na,Nt]),[],1)));
        data.dx(:)=data.dx.^4;                              % modification on lip protrusion
        %data.dx=1.0*data.dx/Nt;                         % modification on vocal tract size
        
        data.Af=max(0,min(64,1.0*(data.Af.^1.0)-.1));   % corrects global alpha-beta values
        idx0=find(data.Af<.5); idx1=1+floor((idx0-1)/size(data.Af,1));
        data.Af(idx0)=data.Af(idx0)+4*max(0,data.P(idx1)'-.75).*(.5/(1-exp(-.5))*(exp(-data.Af(idx0))-exp(-.5)));   % allows for pressure opening of the mouth
        data.Af=max(0,data.Af)+.001; for i=2:Nt, data.Af(:,i)=data.Af(:,i-1).*exp(max(-2,min(1,log(data.Af(:,i))-log(data.Af(:,i-1))))); end; data.Af=max(0,data.Af-.001);  % smooths to avoid clicks 
    end
    opt.Af=data.Af;
end;

switch(TYPE),
    case 'Af',
        % Determine the dimensionality of the area functions
        n = size(data.Af,2);
        
        % Initialize all parameters for vtsynth - [note the transpose]
        Ag0 = data.Ag0';
        AgP = data.AgP';
        F0  = data.F0';
        
        %TAF1 = [linspace(0,data.duration,n)' [0;2 * ones(n-1,1)] -1*ones(n,1)]';
        TAF1 = [F0(1,:)', [0;2 * ones(n-1,1)], -1*ones(n,1)]';
        TAF2 = data.Af;
        
        % call vtsynth to generate the signal
        if 1,
            if isfield(data,'fs'),
                [sig] = vtsynth(size(data.Af,1),Ag0,AgP,F0,TAF1,TAF2,data.dx,data.fs);
            else,
                [sig] = vtsynth(size(data.Af,1),Ag0,AgP,F0,TAF1,TAF2,data.dx);
            end;
        else,   % do it recursively, it might avoid clicks...
            N0=4;
            Na=size(data.Af,1);
            dt=data.duration/Nt; %step size in ms
            Np=round(dt*(Nt-1)*Fs/1000);
            sig=zeros(Np,1);
            idx=(1:Np);
            sF0=(Fs./F0(2,:));
            idxa=1+floor((0:Np-1)/Fs/dt*1000);
            idxb=[1,find(diff(idxa)),Np+1];
            d=ones(1,2); ph=0;
            for n1=1:length(idxb)-1,
                tt=[0,1*sF0(n1+1)*N0/Fs*1000]; %N0*round(idxb(n1+1)-idxb(n1))/Fs*1000];
                sigt=vtsynth(Na,[tt;Ag0(2:end,[1,n1+1])],[tt;AgP(2:end,[n1,n1+1])],[tt;F0(2:end,[n1,n1+1])],[tt;TAF1(2:end,[n1,n1+1])],TAF2(:,[n1,n1+1]),data.dx(:,[n1,n1+1]));
                %sigt=vtsynth(Na,[tt;Ag0(2:end,n1+d)],[tt;AgP(2:end,n1+d)],[tt;F0(2:end,n1+d)],[tt;TAF1(2:end,n1+d)],TAF2(:,n1+d),data.dx(:,n1+d));
                sig(idxb(n1):idxb(n1+1)-1)=sigt(end-ceil(sF0(n1+1))+1+mod(floor(ph*sF0(n1+1))+(1:idxb(n1+1)-idxb(n1))-1,ceil(sF0(n1+1))));
                ph=mod(ph+(idxb(n1+1)-idxb(n1))/sF0(n1+1),1);
            end
            sig=sig/2;
        end
        %sig=sigm(.5*sig);

        AV=interp1(linspace(1,length(sig),n),min(data.Af)>0.1,1:length(sig))';
        sig = sig.*AV;
        
    case 'Tf',
        Tf=10.^(data.Tf/20)-1e-2;
        dt=data.duration/Nt; %step size in ms
        Np=round(dt*(Nt-1)*Fs/1000);
        WS=[20,1]; WS=WS/sum(abs(WS)); % glotal components
        
        % Generates source
        sfilt=hamming(round(Fs/1000*dt)+1); sfilt=sfilt/sum(sfilt);
        F0=repmat(data.F0(:,2)',[round(Fs/1000*dt),1]); F0=convn(F0(:),sfilt,'valid');
        Ag0=repmat(data.Ag0(:,2)',[round(Fs/1000*dt),1]); Ag0=convn(Ag0(:),sfilt,'valid');
        AgP=repmat(data.AgP(:,2)',[round(Fs/1000*dt),1]); AgP=convn(AgP(:),sfilt,'valid');
        phase=cumsum(F0/Fs);
        noise=.5*randn([Np,1])/10;
        Ag0=max(0.1,min(1,Ag0));
        sig0=AgP.*((1-Ag0).*(WS(1)*glotros(0,phase)+WS(2)*glotros(1,phase)) + (Ag0).*noise);
        
        % Vocal cavity filter
        Nf=127;
        %ffilt=real(fft(max(0,data.Tf))); ffilt=ffilt([end-(Nf-1)/2+1:end, 1:(Nf+1)/2],:);
        %ffilt=real(fft((max(0,[zeros(1,Nt);data.Tf;flipud(data.Tf)])))); ffilt=ffilt([end-(Nf-1)/2+1:end, 1:(Nf+1)/2],:);
        time=linspace(1,Nt,Np); timel=floor(time); timer=time-timel;
        ffilt=real(fft((max(0,[zeros(1,Nt);Tf;flipud(Tf)])))); ffilt=ffilt([end-(Nf-1)/2+1:end, 1:(Nf+1)/2],:);
        Sig=sample_window(sig0,Nf,Nf-1,'hanning','same');
        sig=(1-timer).*sum(Sig.*ffilt(:,max(1,min(Nt,timel))),1) + timer.*sum(Sig.*ffilt(:,max(1,min(Nt,timel+1))),1);
        
        sig=2*sig/100;
        %sig=sigm(.5*sig/1000);

end

data.sig=sig;
data.Plot=opt;
