function out=DIVA_Cochlea(action,varargin);
% DIVA_Cochlea Cochlear model
%
% DIVA_Cochlea('init' [,SessionName]);          Initializes the module
% DIVA_Cochlea('save' [,SessionName] );         Saves state
% DIVA_Cochlea('exit');                         Exits the module (without saving)
% DIVA_Cochlea(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_Cochlea('sound',s,fs,delay);
% Sends the sound formants in vector s to the cochlear model
% with a given delay. The module computes the cochlear
% representation (simple spectrogram) of this sound for internal
% purposes, and sends FORMANTS to the AuditoryCortex module.
%
% Current DIVA_AuditoryCortex properties are: (* requires re-initialization)
%   *  delayToAuditoryCortex  : delay (in seconds) for signals to AuditoryCortex
%

  out=[];
  global DIVA_Cochlea_data
  if nargin<1,
    action='init';
  end

  switch(lower(action)),
    %%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%
    %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<2 | isempty(varargin{1}),
      initfile='';
    else,
      initfile=[SessionFolder,'Session_',varargin{1},filesep,mfilename,'.mat'];
    end

    if isempty(initfile) | isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      DIVA_Cochlea_data.params=struct(...
          'fs',10000,...
          'delayToForwardModel',.05,...
          'delayToAuditoryCortex',.005);
    else,
      data=load(initfile,'-mat');
      DIVA_Cochlea_data.params=data.params;
    end
    out={};

   case 'save',
    SessionFolder = strcat(DIVA('SessionFolder'),filesep);
    if nargin<2,
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else,
      initfile=[SessionFolder,'Session_',varargin{1},filesep,mfilename,'.mat'];
    end
    params=DIVA_Cochlea_data.params;
    save(initfile,'params');

   case 'exit',
    clear DIVA_Cochlea_data;

   case 'disp',
    disp(DIVA_Cochlea_data.params);
    out=fieldnames(DIVA_Cochlea_data.params);

    %%%%%%%%%%%%%%%% ACTION %%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

   case 'sound',
    % JSB: Do not compute frequency components, simple spectrogram
    % is all.  Expect formant input, pass on formant output

    s=varargin{1};

    if nargin<4 | isempty(varargin{3}),
      delay=0;
    else,
      delay=varargin{3};
    end

    DIVA_Cochlea_data.params.s = s;
    out = s;
    % compute internal spectrogram for visualization and
    % informational purposes only...Soon!

    % projects output
    if ~nargout,
      DIVA('AuditoryCortex','sound',cat(2,out,nan+zeros(size(out,1),1)),...
           delay+DIVA_Cochlea_data.params.delayToAuditoryCortex);
    end
    
    % Compute spectrogram
   case 'spectrogram',
    y=varargin{1};
    win=hamming(round(0.02*DIVA_Cochlea_data.params.fs));
    
    if(nargout),
      if(strcmp(release_version,'R2006b')),
        [Y,f,t]=spectrogram(filtfilt([1 -0.95],1,y),win,...
                            length(win)-round(0.001*DIVA_Cochlea_data.params.fs),...
                            256,DIVA_Cochlea_data.params.fs);
      else,
        [Y,f,t]=specgram(filtfilt([1 -0.95],1,y),256,DIVA_Cochlea_data.params.fs,...
                         win,length(win)-round(0.001*DIVA_Cochlea_data.params.fs));
      end
      out={Y,f,t};
      %figure,imagesc(abs(Y))
    end

   otherwise,
    if isfield(DIVA_Cochlea_data.params,action),
      if nargin<2,
        out=DIVA_Cochlea_data.params.(action);
      else,
        DIVA_Cochlea_data.params.(action)=varargin{1};
      end
    else,
      warning('DIVA_Cochlea: wrong argument');
    end
  end
