function out=DIVA_ForwardModel(varargin);
% DIVA_ForwardModel DIVA module Forward Model
% The ForwardModel is defined by an HRBF trained on random
% vocaltract 'Babbling'.  
%
% DIVA_ForwardModel('init' [,SessionName]);           Initializes the module
% DIVA_ForwardModel('save' [,SessionName] );          Saves state 
% DIVA_ForwardModel('exit');                          Exits the module (without saving)
% DIVA_ForwardModel(PropertyName [,PropertyValue] )   Reads and writes internal model properties
%
% s=DIVA_ForwardModel('motor',m);                     Read-out values from forward model. m 
%                                                      is the motor command, and s is a 
%                                                      cell array containing the associated 
%                                                      sound and sensory percepts.

% Dependencies: 
%

out=[];
global DIVA_ForwardModel_data
DIVA_ForwardModel_data.current.sound=[];
DIVA_ForwardModel_data.current.sensory=[];
DIVA_ForwardModel_data.current.motor=[];
somethingtoexecute=0;

for indexargin=1:2:nargin,  
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 | isempty(varargin{indexargin+1}), 
      initfile=''; 
    else, 
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
    end
    
    if isempty(initfile) | isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
      DIVA_ForwardModel_data.params=struct(...
          'MaedaSpread',DIVA('VocalTract','MaedaSpread'),...
          'net',load('trainednet4000_040504')); % RBF of stored articulatory
                                                % configurations and auditory consequences

      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
      name=which(mfilename);
      if ~isempty(dir([name(1:end-2),'.mat'])), 
        Data = load([name(1:end-2),'.mat']);
      else 
        Data = struct('paramsSound',[],'paramsSensory',[]); 
      end
      DIVA_ForwardModel_data.params.Data = Data;
      clear Data 
    else,
      data = load(initfile,'-mat');
      DIVA_ForwardModel_data.params = data.params;
    end
    
    % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%               
    DIVA_ForwardModel_data.current=struct(...
        'motor',[]);
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    out={'motor'};
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   case 'save',
    SessionFolder = strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1, 
      initfile = [SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else, 
      initfile = [SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
    end
    params = DIVA_ForwardModel_data.params;
    save(initfile,'params'); 
    
   case 'exit',
    clear DIVA_ForwardModel_data;
    % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   case 'disp',
    disp(DIVA_ForwardModel_data.params);
    out=fieldnames(DIVA_ForwardModel_data.params);
    
   case 'motor',
    if ~nargout,
      DIVA_ForwardModel_data.current.motor = varargin{indexargin+1};
    else, % reads-out internalmodel value(s)
      % Compute Motor to Auditory transformation
      
      Ninput = size(varargin{indexargin+1},2);
      for idx = 1:Ninput,
        out(:,idx) = simrbf(DIVA_ForwardModel_data.params.net.net,...
                            varargin{indexargin+1}(:,idx));
      end      
    end
            
   otherwise,
    if isfield(DIVA_ForwardModel_data.params,varargin{indexargin}),
      if indexargin==nargin, 
        out=DIVA_ForwardModel_data.params.(varargin{indexargin}); 
      else, 
        DIVA_ForwardModel_data.params.(varargin{indexargin})=varargin{indexargin+1}; 
      end
    else, 
      warning('DIVA_ForwardModel: wrong argument'); 
    end
  end
end


