function out=DIVA_ModelStatePlot(varargin);
% DIVA_ModelStatePlot DIVA architecture plotting module.  Displays
% a plot of active module components complete with time point of activity
%
% DIVA_ModelStatePlot('init' [,SessionName]);          Initializes the module
% DIVA_ModelStatePlot('save' [,SessionName] );         Saves state 
% DIVA_ModelStatePlot('exit');                         Exits the module (without saving)
% DIVA_ModelStatePlot(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_ModelStatePlot(ChannelName1,ChannelValue1,...
%               ChannelName2,ChannelValue2,...); Executes module process(es)
%

% Dependencies: 
%

out=[];
global DIVA_ModelStatePlot_data

for indexargin=1:2:nargin,  
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 | isempty(varargin{indexargin+1}), 
      initfile='';
    else, 
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
    end
    if isempty(initfile) | isempty(dir(initfile)),
      % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
      DIVA_ModelStatePlot_data.params=struct(...
          'historylength',200,...
          'plotdata',{... % info for plotting known channels: x coordinate, y coordinate, point type
              {... 
                  340+(0:30:90),245+0*(0:30:90),'ks',...     % VocalTract motor
                  780+.7*(0:30:90),135+.7*(0:30:90),'ks',... % AuditoryCortex target
                  500+(0:30:90),75+0*(0:30:90),'ks',...      % AuditoryCortex sound
                  830-.7*(0:30:90),310+.7*(0:30:90),'ks',... % SomatosensoryCortex target
                  500+(0:30:90),420+0*(0:30:90),'ks',...     % SomatosensoryCortex somatosensory
                  640+(0:30:90),245+0*(0:30:90),'ks',...     % MotorCortex target
                  630-.7*(0:30:90),135+.7*(0:30:90),'ks',... % MotorCortex error_sound
                  575+.7*(0:30:90),310+.7*(0:30:90),'ks',... % MotorCortex error_somatosensory
                  820+(0:30:90),75+0*(0:30:90),'ks',...      % AuditoryCortexCategorical sound
                  940-.7*(0:30:90),135+.7*(0:30:90),'ks'...  % SoundMap target
              }},...
          'plotdatanames',{{... % names of module channels in the plot window
              'VocalTract','motor';
              'AuditoryCortex','target';
              'AuditoryCortex','sound';
              'SomatosensoryCortex','target';
              'SomatosensoryCortex','somatosensory';
              'MotorCortex','target';
              'MotorCortex','error_sound';
              'MotorCortex','error_somatosensory';
              'AuditoryCortexCategorical','sound';
              'SoundMap','target'}});
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
    else,
      data=load(initfile,'-mat');
      DIVA_ModelStatePlot_data.params=data.params;
    end
    % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%               
    h=findobj('name','DIVA model state plot');
    if isempty(h), 
      h=figure('name','DIVA model state plot',...
               'numbertitle','off',...
               'units','norm',...
               'position',[.5,.5,.45,.35],...
               'menu','none',...
               'doublebuffer','on',...
               'color','w',...
               'deletefcn','DIVA(''exit'');'); 
    else, 
      figure(h); 
    end
    
    DIVA_ModelStatePlot_data.current=struct(...
        'figurehandle',h,...
        'figuredata',imread([mfilename,'.tif']),...
        'Now',DIVA('Now'),...
        'markers','^>v<'); 
    
    subplot(211); 
    imagesc(DIVA_ModelStatePlot_data.current.figuredata); 
    axis equal; 
    axis tight; 
    axis off; 
    hold on; 
    set(gca,'units','norm','position',[0,.5,1,.5]);
    DIVA_ModelStatePlot_data.current.plotdata = ...
        plot(DIVA_ModelStatePlot_data.params.plotdata{:}); 
    hold off;
    
    set(DIVA_ModelStatePlot_data.current.plotdata,'markerfacecolor','k','markersize',1);
    DIVA_ModelStatePlot_data.current.plotdatanames = ...
        DIVA_ModelStatePlot_data.params.plotdatanames;
    DIVA_ModelStatePlot_data.current.history = ...
        zeros([size(DIVA_ModelStatePlot_data.current.plotdatanames,1),...
               DIVA_ModelStatePlot_data.params.historylength]);
    
    subplot(212); 
    DIVA_ModelStatePlot_data.current.historydata = ...
        imagesc(DIVA_ModelStatePlot_data.current.history); 
    axis tight; 
    for n1=1:size(DIVA_ModelStatePlot_data.current.plotdatanames,1), 
      ticklabel{n1} = strcat(DIVA_ModelStatePlot_data.current.plotdatanames{n1,1},' -  ',...
                             DIVA_ModelStatePlot_data.current.plotdatanames{n1,2}); 
    end
    
    set(gca,'units','norm','position',[.05,.05,.65,.45],...
            'clim',[0,1],'xtick',0:10:DIVA_ModelStatePlot_data.params.historylength,...
            'ytick',1:size(DIVA_ModelStatePlot_data.current.plotdatanames,1),...
            'yticklabel',ticklabel,'yaxislocation','right',...
            'fontsize',6,'box','off','xticklabel',[]); 
    xlabel('==== TIME ===>')
    set(DIVA_ModelStatePlot_data.current.historydata,'erasemode','none'); 
    colormap([1,1,1;.8,.8,.8]);
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
    out={}; 
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   case 'save',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1, 
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat']; 
    else, 
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
    end
    params=DIVA_ModelStatePlot_data.params;
    save(initfile,'params'); 
    
   case 'exit',
    if ishandle(DIVA_ModelStatePlot_data.current.figurehandle), 
      set(DIVA_ModelStatePlot_data.current.figurehandle,'deletefcn','');
      delete(DIVA_ModelStatePlot_data.current.figurehandle); 
    end
    clear DIVA_ModelStatePlot_data;
    % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   case 'disp',
    disp(DIVA_ModelStatePlot_data.params);
    out=fieldnames(DIVA_ModelStatePlot_data.params);
    
    % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   case {DIVA_ModelStatePlot_data.current.plotdatanames{:,1}},
    Now=DIVA('Now');
    if Now~=DIVA_ModelStatePlot_data.current.Now, 
      set(DIVA_ModelStatePlot_data.current.historydata,...
          'cdata',DIVA_ModelStatePlot_data.current.history);
      drawnow; 
      DIVA_ModelStatePlot_data.current.history(:,1:end-1) = ...
          DIVA_ModelStatePlot_data.current.history(:,2:end); 
      DIVA_ModelStatePlot_data.current.history(:,end)=0;
      set(DIVA_ModelStatePlot_data.current.plotdata,'markersize',1); 
    end
    index=intersect(...
        strmatch(varargin{indexargin},...
                 {DIVA_ModelStatePlot_data.current.plotdatanames{:,1}},'exact'),...
        strmatch(varargin{indexargin+1},...
                 {DIVA_ModelStatePlot_data.current.plotdatanames{:,2}},'exact'));
    if ~isempty(index), 
      DIVA_ModelStatePlot_data.current.history(index,end)=1;
      set(DIVA_ModelStatePlot_data.current.plotdata(index),...
          'markersize',5,...
          'marker',DIVA_ModelStatePlot_data.current.markers(1+mod(Now+min(index),4))); 
    end
    DIVA_ModelStatePlot_data.current.Now=Now;
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   otherwise,
    if isfield(DIVA_ModelStatePlot_data.params,varargin{indexargin}),
      if indexargin==nargin, 
        out=DIVA_ModelStatePlot_data.params.(varargin{indexargin}); 
      else, 
        DIVA_ModelStatePlot_data.params.(varargin{indexargin})=varargin{indexargin+1}; 
      end
    else, 
      warning('DIVA_ModelStatePlot: wrong argument'); 
    end
  end
end

