function out=DIVA_SensoryChannel(varargin);
% DIVA_SensoryChannel DIVA Sensory-Channel module
%
% This module incorporates any sensory transformations between
% the sensory percept produced by the vocal tract configuration
% and the actual sensory configuration perceived by the sensory module.
%
% DIVA_SensoryChannel('init' [,SessionName]);          Initializes the module
% DIVA_SensoryChannel('save' [,SessionName] );         Saves state 
% DIVA_SensoryChannel('exit');                         Exits the module (without saving)
% DIVA_SensoryChannel(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_SensoryChannel(ChannelName1,ChannelValue1,...
%               ChannelName2,ChannelValue2,...); Executes module process(es)
%

% Dependencies: 
%

out=[];
global DIVA_SensoryChannel_data

indexargin=1; 
switch(varargin{indexargin}),
 case 'init',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<indexargin+1 | isempty(varargin{indexargin+1}), 
    initfile=''; 
  else, 
    initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
  end
  if isempty(initfile) || isempty(dir(initfile)),
    disp([mfilename, ' : Defining new session...']);
    % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
    DIVA_SensoryChannel_data.params=struct(...
        'delayToSensory',.0);  
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  else,
    data=load(initfile,'-mat');
    DIVA_SensoryChannel_data.params=data.params;
  end
  % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%               
  DIVA_SensoryChannel_data.params.current=struct(...
      'configuration',[],...
      'delay',[]);   
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  out={};   
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 case 'save',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<indexargin+1, 
    initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat']; 
  else, 
    initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat']; 
  end
  params=DIVA_SensoryChannel_data.params;
  save(initfile,'params'); 
  
 case 'exit',
  clear DIVA_SensoryChannel_data;
  % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 case 'disp',
  disp(DIVA_SensoryChannel_data.params);
  out=fieldnames(DIVA_SensoryChannel_data.params);
  
  % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Note: use DIVA(ModuleName,ChannelName,data,delay) to comunicate with other DIVA modules
 case 'configuration',
  DIVA_SensoryChannel_data.current.configuration = varargin{indexargin+1};
  if nargin<indexargin+2 | isempty(varargin{indexargin+2}), 
    DIVA_SensoryChannel_data.current.delay = 0; 
  else, 
    DIVA_SensoryChannel_data.current.delay = varargin{indexargin+2}; 
  end
  DIVA_SensoryChannel_data.current.delay = ...
      DIVA_SensoryChannel_data.current.delay + ...
      DIVA_SensoryChannel_data.params.delayToSensory;
  
  if ~nargout,
    DIVA('Sensory','configuration',...
         DIVA_SensoryChannel_data.current.configuration,...
         DIVA_SensoryChannel_data.current.delay); 
  else, 
    out={DIVA_SensoryChannel_data.current.configuration}; 
  end
  
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 otherwise,
  if isfield(DIVA_SensoryChannel_data.params,varargin{indexargin}),
    if indexargin==nargin, 
      out=DIVA_SensoryChannel_data.params.(varargin{indexargin}); 
    else, 
      DIVA_SensoryChannel_data.params.(varargin{indexargin})=varargin{indexargin+1}; 
    end
  else, 
    warning('DIVA_SensoryChannel: wrong argument'); 
  end
end
