function out=DIVA_SoundChannel(varargin);
% DIVA_SoundChannel DIVA Sound-Channel module.
%
% This module incorporates any sound transformations due to the
% channel between the sound produced by the model vocal tract
% and the sound perceived by the model cochlea.
%
% DIVA_SoundChannel('init' [,SessionName]);          Initializes the module
% DIVA_SoundChannel('save' [,SessionName] );         Saves state
% DIVA_SoundChannel('exit');                         Exits the module (without saving)
% DIVA_SoundChannel(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_Cochlea('sound',s,fs,delay);
% DIVA_Cochlea('noise',s,fs);
% Forwards its input to the Cochlea module with a given delay (defined by
% the 'delayToCochlea' module property)

% Dependencies:
%

out=[];
global DIVA_SoundChannel_data

indexargin=1;
switch(varargin{indexargin}),
 case 'init',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<indexargin+1 | isempty(varargin{indexargin+1}),
    initfile='';
  else,
    initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
  end

  if isempty(initfile) || isempty(dir(initfile)),
    disp([mfilename, ' : Defining new session...']);
    % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    DIVA_SoundChannel_data.params=struct(...
        'fs',DIVA('Cochlea','fs'),...
        'delayToCochlea',.0);
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  else,
    data=load(initfile,'-mat');
    DIVA_SoundChannel_data.params=data.params;
  end
  % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  DIVA_SoundChannel_data.params.current=struct(...
      'sound',[],...
      'fs',[],...
      'delay',[]);
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  out={};
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 case 'save',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<indexargin+1,
    initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
  else,
    initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
  end
  params=DIVA_SoundChannel_data.params;
  save(initfile,'params');

 case 'exit',
  clear DIVA_SoundChannel_data;
  % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 case 'disp',
  disp(DIVA_SoundChannel_data.params);
  out=fieldnames(DIVA_SoundChannel_data.params);

  % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Note: use DIVA(ModuleName,ChannelName,data,delay) to comunicate with other DIVA modules
 case {'sound','noise'},
  DIVA_SoundChannel_data.current.sound=varargin{indexargin+1};
  if nargin<indexargin+2 | isempty(varargin{indexargin+2}),
    DIVA_SoundChannel_data.current.fs=DIVA_SoundChannel_data.params.fs;
  else,
    DIVA_SoundChannel_data.current.fs=varargin{indexargin+2};
  end

  if nargin<indexargin+3 | isempty(varargin{indexargin+3}),
    DIVA_SoundChannel_data.current.delay=0;
  else,
    DIVA_SoundChannel_data.current.delay=varargin{indexargin+3};
  end

  DIVA_SoundChannel_data.current.delay=...
      DIVA_SoundChannel_data.current.delay+...
      DIVA_SoundChannel_data.params.delayToCochlea;

  if ~nargout,
    DIVA('Cochlea',varargin{indexargin},DIVA_SoundChannel_data.current.sound,...
         DIVA_SoundChannel_data.current.fs,DIVA_SoundChannel_data.current.delay);
  else,
    out={DIVA_SoundChannel_data.current.sound};
  end
  % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 otherwise,
  if isfield(DIVA_SoundChannel_data.params,varargin{indexargin}),
    if indexargin==nargin,
      out=DIVA_SoundChannel_data.params.(varargin{indexargin});
    else,
      DIVA_SoundChannel_data.params.(varargin{indexargin})=varargin{indexargin+1};
    end
  else,
    warning('DIVA_SoundChannel: wrong argument');
  end
end

