function out=DIVA_SoundMap(varargin);
% DIVA_SoundMAP Speech initiation module
%
% This model initiates the production of phonemic segments,
% sending an index to the desired phonemic target to MotorCortex,
% AuditoryCortex and SomatosensoryCortex.
%
% DIVA_SoundMap('init' [,SessionName]);          Initializes the module
% DIVA_SoundMap('save' [,SessionName] );         Saves state
% DIVA_SoundMap('exit');                         Exits the module (without saving)
% DIVA_SoundMap(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_SoundMap('target',targetindex);
% Projects auditory, somatosensory, and motor associations of
% a given phonemic target.
%
% Current DIVA_SoundMap properties are: (* requires re-initialization)
%  *  delayToMotorCortex          : delay (in seconds) for signals to MotorCortex
%

% Dependencies:
%

out=[];
global DIVA_SoundMap_data

for indexargin=1:2:nargin,
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 | isempty(varargin{indexargin+1}),
      initfile='';
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    if isempty(initfile) | isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      DIVA_SoundMap_data.params=struct(...
          'delayToMotorCortex',.005);
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    else,
      data=load(initfile,'-mat');
      DIVA_SoundMap_data.params=data.params;
    end
    % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    DIVA_SoundMap_data.params.delayToSomatosensoryCortex=...
        DIVA_SoundMap_data.params.delayToMotorCortex+...
        DIVA('MotorCortex','delayToVocalTract')+...
        DIVA('VocalTract','delayToSensoryChannel')+0*...
        DIVA('SensoryChannel','delayToSensory')+...
        DIVA('Sensory','delayToSomatosensoryCortex');

    DIVA_SoundMap_data.params.delayToAuditoryCortex=...
        DIVA_SoundMap_data.params.delayToMotorCortex+...
        DIVA('MotorCortex','delayToVocalTract')+...
        DIVA('VocalTract','delayToSoundChannel')+0*...
        DIVA('SoundChannel','delayToCochlea')+...
        DIVA('Cochlea','delayToAuditoryCortex');

    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    out={'target'};
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   case 'save',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1,
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    params=DIVA_SoundMap_data.params;
    save(initfile,'params');

   case 'exit',
    clear DIVA_SoundMap_data;
    % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   case 'disp',
    disp(DIVA_SoundMap_data.params);
    out=fieldnames(DIVA_SoundMap_data.params);

    % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   case 'target',
    % Sends target commands to MotorCortex, AuditoryCortex, and SomatosensoryCortex
    targetindex=varargin{indexargin+1};
    DIVA('MotorCortex','target',targetindex,...
         DIVA_SoundMap_data.params.delayToMotorCortex);
    DIVA('SomatosensoryCortex','target',targetindex,...
         DIVA_SoundMap_data.params.delayToSomatosensoryCortex);
    DIVA('AuditoryCortex','target',targetindex,...
         DIVA_SoundMap_data.params.delayToAuditoryCortex);
    if ~nargout,
      DIVA('ModelStatePlot','SoundMap','target');
    end

    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   otherwise,
    if isfield(DIVA_SoundMap_data.params,varargin{indexargin}),
      if indexargin==nargin,
        out=DIVA_SoundMap_data.params.(varargin{indexargin});
      else,
        DIVA_SoundMap_data.params.(varargin{indexargin})=varargin{indexargin+1};
      end
    else,
      warning('DIVA_SoundMap: wrong argument');
    end
  end
end

