function gui_all_params_cbk(src,eventdata)
% gui_all_params_cbk(src,eventdata)
% 
% Callback function for Edit All menu item
%

% 2007-03-25: (JSB) Rewritten without uipanel, was crashing other machines,
% especially non-Matlab version 7+ computers
%

  h=figure('name','DIVA Parameter Editor','numbertitle','off',...
           'units','pixels','position',[100 100 800 600],'menu','none',...
           'doublebuffer','on','color',[0.5 0.5 0.5]);

  %-----------------DIVA Main Parameters--------------------------%

  %diva_area=[0.05 0.525 0.225-0.05 0.975-0.525];
  diva_area = [0.025 0.675 0.45 0.975-0.675];
  
  diva_panel=uicontrol('style','frame','units','norm',...
    'position',diva_area,...
    'backgroundcolor',[0.7 0.7 0.7]);
  
  % height = height of text and edits, 
  %          factor 0.8 -> effective frame height multiplier
  height = (0.8*diva_area(4))/4; % 0.2 * height of frame
  % spacer = space between text/edits and each other as well as bounds
  spacer = (0.8*diva_area(4)-2*height)/3;
  % width  = width = text, 1-width = width of edits
  width  = (2/3)*diva_area(3)/2;
  
  labels.diva_title=...
      uicontrol('style','text',...
                'parent',h,'string','DIVA',...
                'FontUnits','pixels','FontSize',16,...
                'FontWeight','bold','Units','norm','position',...
                [diva_area(1)+0.0075 diva_area(2)+0.85*diva_area(4) ...
                 3*diva_area(3)/5 0.1*diva_area(4)]);
  
  labels.diva_timestep=...
      uicontrol('style','text',...
                'parent',h,'string','Time Step',...
                'FontUnits','points','FontSize',13,...
                'Units','norm','Position',...
                [diva_area(1)+0.001 diva_area(2)+(2*spacer+1*height) width height]);

  labels.diva_plotstep=...
      uicontrol('style','text',...
                'parent',h,'string','Plot Step',...
                'FontUnits','points','FontSize',13,...
                'Units','norm','Position',...
                [diva_area(1)+0.001 diva_area(2)+(1*spacer+0*height) width height]);
  
  labels.diva_storagetime=...
      uicontrol('style','text',...
                'parent',h,'string',...
                sprintf('%s\n%s','Storage', 'Time'),...
                'FontUnits','points','FontSize',13,...
                'Units','norm','Position',...
                [diva_area(1)+0.001+diva_area(3)*.5 ...
                 diva_area(2)+(2*spacer+1*height-height/2) ...
                 width 2*height]);
               
  checkbox.diva_blockmode=...
      uicontrol('style','checkbox',...
                'min',0,'max',1,...
                'UserData',{'DIVA','BlockMode',[0 1]},...
                'parent',h,'string','Block Mode',...
                'FontUnits','points','FontSize',13,...
                'units','norm','Position',...
                [diva_area(1)+0.001+diva_area(3)*.5 diva_area(2)+(1*spacer+0*height) ...
                 diva_area(3)/2-0.002 height],...
                'BackgroundColor',[0.7 0.7 0.7]);

  edits.diva_timestep=...
      uicontrol('style','edit','UserData',...
                {'DIVA','TimeStep',[1 5]},...
                'parent',h,'FontUnits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [diva_area(1)+width+0.001 ...
                 diva_area(2)+2*spacer+1*height+0.25*height ...
                 diva_area(3)*0.5-width-0.01 height]);

  edits.diva_plotstep=...
      uicontrol('style','edit','UserData',{'DIVA','PlotStep',[1 50]},...
                'parent',h,'FontUnits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [diva_area(1)+width+0.001 ...
                 diva_area(2)+1*spacer+0*height+0.25*height...
                 diva_area(3)*0.5-width-0.01 height]);

  edits.diva_storagetime=...
      uicontrol('style','edit','UserData',{'DIVA','StorageTime',[1 25]},...
                'parent',h,'FontUnits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [diva_area(1)+diva_area(3)*.5+width+0.001 ...
                 diva_area(2)+2*spacer+1*height+0.25*height ...
                 diva_area(3)*0.5-width-0.01 height]);

  %-------------------------Somatosensory Parameters---------------------------%

  somato_area=[0.525 0.75 0.45 0.975-0.75];    
  
  somato_panel=uicontrol('style','frame','parent',h,...
                         'units','norm','position',somato_area,...
                         'backgroundcolor',[0.7 0.7 0.7]);

  labels.somato_title=...
      uicontrol('style','text',...
                'parent',h,'string','Somatosensory Cortex',...
                'FontUnits','points','FontSize',12,...
                'FontWeight','bold','units','norm','position',...
                [somato_area(1)+0.0075 somato_area(2)+0.825*somato_area(4) ...
                 17*somato_area(3)/19 0.15*somato_area(4)]);
  
  labels.somato_eps_learn=...
      uicontrol('style','text',...
                'parent',h,...
                'string',sprintf('%s\n%s','Learning','Rate'),...
                'FontUnits','points',...
                'Units','norm','FontSize',13,...
                'Position',...
                [somato_area(1)+0.002 diva_area(2)+(2*spacer+1*height-height/2) ...
                 width 2*height]);

  edits.somato_eps_learn=...
      uicontrol('style','edit','parent',h,...
                'UserData',...
                {'SomatosensoryCortex','eps_learn',[0 1]},...
                'FontUnits','norm','callback',@verify,...
                'units','norm','position',...
                [somato_area(1)+width+0.0025 ...
                 diva_area(2)+(2*spacer+1*height)+0.25*height ...
                 somato_area(3)*0.5-width-0.01 height]);
                                                    

  %------------------------------Motor Parameters-------------------------------%

  %-------------------------Motor: Feedforward command--------------------------%
  
  motor_ff_area=[0.025 0.175 0.45 0.45];
  motor_ff_panel=uicontrol('style','frame','parent',h,...
                           'units','norm','position',motor_ff_area,...
                           'backgroundcolor',[0.7 0.7 0.7]);
  
  labels.motor_ff_title=...
      uicontrol('style','text','parent',h,...
                'string','Motor Cortex: Feedforward Command',...
                'fontunits','points','fontsize',13,...
                'fontweight','bold','units','norm','position',...
                [motor_ff_area(1)+0.0075 motor_ff_area(2)+0.85*motor_ff_area(4) ...
                 motor_ff_area(3)-0.015 0.1*motor_ff_area(4)]);
  
  height = (motor_ff_area(4)-.2*motor_ff_area(4))/8;
  spacer = (motor_ff_area(4)-.2*motor_ff_area(4)-4*height)/5;
  width  = motor_ff_area(3)*(3/4);
  
  labels.motor_ff_learn=...
      uicontrol('style','text','parent',h,...
                'string','Learning Weight',...
                'Fontunits','points','fontsize',13,...
                'units','norm','position',...
                [motor_ff_area(1)+0.0075 motor_ff_area(2)+4*spacer+3*height ...
                 width-0.0075 height]);

  labels.motor_ff_learn_a=...
      uicontrol('style','text','parent',h,...
                'string','Auditory',...
                'Fontunits','points','fontsize',12,...
                'units','norm','position',...
                [motor_ff_area(1)+0.0075*3 ...
                 motor_ff_area(2)+3*spacer+2*height ...
                 (2/3)*(0.45*motor_ff_area(3)) height]);

  labels.motor_ff_learn_s=...
      uicontrol('style','text','parent',h,...
                'string','Somatosensory',...
                'Fontunits','points','fontsize',12,...
                'units','norm','position',...
                [motor_ff_area(1)+0.45*motor_ff_area(3) ...
                 motor_ff_area(2)+3*spacer+2*height ...
                 0.55*motor_ff_area(3)*(3/4) height]);
  
  edits.motor_ff_learn=...
      uicontrol('style','edit','parent',h,...
                'userdata',{'MotorCortex','WeightLearn',[0,1]},...
                'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'position',...
                [motor_ff_area(1)+0.45*motor_ff_area(3)+0.55*motor_ff_area(3)*(3/4) ...
                 motor_ff_area(2)+(4*spacer+3*height) ...
                 0.55*motor_ff_area(3)*(1/4)-0.005 ...
                 1.25*height-0.005]);  
  
  edits.motor_ff_learn_a=...
      uicontrol('style','edit','parent',h,...
                'userdata',{'MotorCortex','WeightLearn_sound',[0,1]},...
                'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'position',...
                [motor_ff_area(1)+(2/3)*(0.45*motor_ff_area(3)) ...
                 motor_ff_area(2)+(3*spacer+2*height) ...
                 (1/3)*(0.45*motor_ff_area(3))-0.005 ...
                 1.25*height-0.005]);
  
  edits.motor_ff_learn_s=...
      uicontrol('style','edit','parent',h,...
                'userdata',{'MotorCortex','WeightLearn_somatosensory',[0,1]},...
                'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'position',...
                [motor_ff_area(1)+0.45*motor_ff_area(3)+0.55*motor_ff_area(3)*(3/4) ...
                 motor_ff_area(2)+(3*spacer+2*height) ...
                 0.55*motor_ff_area(3)*(1/4)-0.005 ...
                 1.25*height-0.005]);  
  
  %------------------------Motor: Inst. Command---------------------------------%
  
  motor_cmd_area=[0.525 0.175 0.45 0.45];
  motor_cmd_panel=uicontrol('style','frame','parent',h,...
                           'units','norm','position',motor_cmd_area,...
                           'backgroundcolor',[0.7 0.7 0.7]);
  labels.motor_cmd_title=...
      uicontrol('style','text','parent',h,...
                'string','Motor Cortex: Instantaneous Motor Command',...
                'fontunits','points','fontsize',13,...
                'fontweight','bold','units','norm','position',...
                [motor_cmd_area(1)+0.0075 motor_cmd_area(2)+0.825*motor_cmd_area(4) ...
                 motor_cmd_area(3)-0.015 0.15*motor_cmd_area(4)]);

  height = (motor_cmd_area(4)-.2*motor_cmd_area(4))/8;
  spacer = (motor_cmd_area(4)-.2*motor_cmd_area(4)-4*height)/5;
  width  = (9/16)*motor_cmd_area(3)*(3/4);
  
  labels.motor_contrib_ff=...
      uicontrol('style','text',...
                'parent',h,'string','FeedForward',...
                'FontUnits','points','FontSize',13,...
                'Units','norm','Position',...
                [motor_cmd_area(1)+0.0075 motor_cmd_area(2)+(4*spacer+3*height) ...
                 width-0.0075 height]);
  
  labels.motor_contrib_fb=...
      uicontrol('style','text',...
                'parent',h,'string','FeedBack',...
                'FontUnits','points','FontSize',13,...
                'Units','norm','Position',...
                [motor_cmd_area(1)+0.0075 motor_cmd_area(2)+(3*spacer+2*height) ...
                 width-0.0075 height]);
  
  labels.motor_contrib_fba=...
      uicontrol('style','text',...
                'parent',h,'string','Auditory',...
                'FontUnits','points','FontSize',12,...
                'Units','norm','Position',...
                [motor_cmd_area(1)+0.0075*3 motor_cmd_area(2)+(2*spacer+1*height) ...
                 width-.0075*3 height]);
    
  labels.motor_contrib_fbs=...
      uicontrol('style','text',...
                'parent',h,'string','Somatosensory',...
                'FontUnits','points','FontSize',12,...
                'Units','norm','Position',...
                [motor_cmd_area(1)+0.0075*3 motor_cmd_area(2)+(1*spacer+0*height) ...
                 width-0.0075*3 height]);
  
  edits.motor_contrib_ff=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','WeightFeedForward',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [motor_cmd_area(1)+width ...
                 motor_cmd_area(2)+(4*spacer+3*height) ...
                 (9/16)*motor_cmd_area(3)-width 1.25*height-0.005]);

  edits.motor_contrib_fb=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','WeightFeedBack',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [motor_cmd_area(1)+width ...
                 motor_cmd_area(2)+(3*spacer+2*height) ...
                 (9/16)*motor_cmd_area(3)-width 1.25*height-0.005]);

  edits.motor_contrib_fba=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','WeightFeedBack_sound',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [motor_cmd_area(1)+width ...
                 motor_cmd_area(2)+(2*spacer+1*height) ...
                 (9/16)*motor_cmd_area(3)-width 1.25*height-0.005]);

  edits.motor_contrib_fbs=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','WeightFeedBack_somatosensory',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'Position',...
                [motor_cmd_area(1)+width ...
                 motor_cmd_area(2)+(1*spacer+0*height) ...
                 (9/16)*motor_cmd_area(3)-width 1.25*height-0.005]);
  
  width = (6/16)*motor_cmd_area(3)*(2/3);
  
  labels.motor_integrate=...
      uicontrol('style','text',...
                'parent',h,'string','Int. Rate',...
                'Fontunits','points','fontsize',13,...
                'units','norm','position',...
                [motor_cmd_area(1)+(9/16)*motor_cmd_area(3)+0.001 ...
                 motor_cmd_area(2)+(4*spacer+3*height) ...
                 width height]);
  
  labels.motor_decay=...
      uicontrol('style','text',...
                'parent',h,'string','Decay Rate',...
                'Fontunits','points','fontsize',13,...
                'units','norm','position',...
                [motor_cmd_area(1)+(9/16)*motor_cmd_area(3)+0.001 ...
                 motor_cmd_area(2)+(3*spacer+2*height) ...
                 width height]);
  
  edits.motor_integrate=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','eps_motor',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'position',...
                [motor_cmd_area(1)+(9/16)*motor_cmd_area(3)+width+0.005 ...
                 motor_cmd_area(2)+(4*spacer+3*height) ...
                 (6/16)*motor_cmd_area(3)*(1/3) ...
                 1.25*height-0.005]);
  
  edits.motor_decay=...
      uicontrol('style','edit','userdata',...
                {'MotorCortex','eps_decay',[0 1]},...
                'parent',h,'fontunits','norm',...
                'callback',@verify,'units','norm',...
                'position',...
                [motor_cmd_area(1)+(9/16)*motor_cmd_area(3)+width+0.005 ...
                 motor_cmd_area(2)+(3*spacer+2*height) ...
                 (6/16)*motor_cmd_area(3)*(1/3) ...
                 1.25*height-0.005]);

  %----------------------------Set/cancel-------------------------------------%

  btn.ok=uicontrol('parent',h,'style','pushbutton','units','norm',...
                   'position',[.8 .1 .05 .05],'FontUnits','norm',...
                   'FontSize',.45,'String','OK',...
                   'callback',{@ok_cbk,edits,checkbox});
  btn.cancel=uicontrol('parent',h,'style','pushbutton','units','norm',...
                       'position',[.85 .1 .1 .05],'FontUnits','norm',...
                       'FontSize',.45,'String','Cancel',...
                       'callback',@cancel_cbk);

  % ----------------------- Initialize all elements----------------------%

  set(h,'resize','off');
  
  fnames=fieldnames(labels);
  for i=1:length(fnames)
    set(labels.(fnames{i}),'BackgroundColor',[0.7 0.7 0.7],...
                      'HorizontalAlignment','left');
  end
  %set(labels.diva_title,'HorizontalAlignment','center');
  %set(labels.somato_title,'HorizontalAlignment','center');
  
  fnames=fieldnames(edits);
  for i=1:length(fnames)
    set(edits.(fnames{i}),'FontSize',0.4,'callback',@verify);
    out=get(edits.(fnames{i}),'UserData');
    if(~isempty(out) && length(out)>=2),
      setParamsGUI(edits.(fnames{i}),out);
    end
  end

  fnames=fieldnames(checkbox);
  for i=1:length(fnames)
    set(checkbox.(fnames{i}),'callback',@verify);
    out=get(checkbox.(fnames{i}),'UserData');
    if(~isempty(out) && length(out)>=2),
      setParamsGUI(checkbox.(fnames{i}),out);
    end
  end

%----------------------------Functions--------------------------%

function verify(src,eventdata)
% Verify user input is in valid range and of appropriate type (integer/real)
% Also provide correct formatting for data (1,2,3 decimals or integer)

  if(~isempty(get(src,'UserData')))
    out=get(src,'UserData');
    if(length(out)==3)
      range=out{3};
      if(strcmp(get(src,'style'),'checkbox')),
      else
        if(strcmp(out{2},'PlotStep') || strcmp(out{2},'StorageTime') || ...
           strcmp(out{2},'ntimepoints') || strcmp(out{2},'TimeStep'))
          % must be integer valued
          if((mod(str2double(get(src,'string')),10)<1 && ...
              mod(str2double(get(src,'string')),10)>0) ||...
             str2double(get(src,'string'))==0)
            errordlg(sprintf('Entered value must be integer'),'Warning...');
            setParamsGUI(src,out);
            return;
          end
        end
        % check to see if need 3 dec
        if(any(mod(range*100,100)<1 & mod(range*100,100)>0))
          rangestr=sprintf('[%1.3f %1.3f]',range(1),range(2));
        elseif(any(mod(range*10,10)<1 & mod(range*10,10)>0))   % check for 2 dec
          rangestr=sprintf('[%1.2f %1.2f]',range(1),range(2));
        elseif(any(mod(range,1)<1 & mod(range,1)>0))           % check for 1 dec
          rangestr=sprintf('[%1.1f %1.1f]',range(1),range(2));
        elseif(all(mod(range,1)==0))                           % check for int
          rangestr=sprintf('[%d %d]',range(1),range(2));
        end
        if(str2double(get(src,'string'))>range(2) || str2double(get(src,'string'))<range(1))
          errordlg(sprintf('Entered value must be in range %s',rangestr),...
                   'Warning...','modal');
          setParamsGUI(src,out);
        end
      end
    end
  end

  %************************************************************%
function setParamsGUI(src,options)
  if(strcmp(get(src,'style'),'checkbox'))
    if(strcmp(options{1},'DIVA')),
      set(src,'value',DIVA(options{2}));
    else
      set(src,'value',DIVA(options{1},options{2}));
    end
  else
    if(strcmp(options{1},'DIVA'))
      set(src,'string',num2str(DIVA(options{2})));
    else
      set(src,'string',num2str(DIVA(options{1},options{2})));
    end
  end

  %************************************************************%
function setParamsDIVA(options,value)
  if(ischar(value))
    value=str2double(value);
  end
  if(strcmp(options{2},'TimeStep'))
    value=value/1000;
  end
  if(strcmp(options{1},'DIVA'))
    DIVA(options{2},value);
  else
    DIVA(options{1},options{2},value);
  end

  %************************************************************%
function ok_cbk(src,eventdata,edits,checkbox)
  fnames=fieldnames(edits);
  for i=1:length(fnames),
    setParamsDIVA(get(edits.(fnames{i}),'UserData'),get(edits.(fnames{i}),'string'));
  end
  fnames=fieldnames(checkbox);
  for i=1:length(fnames),
    setParamsDIVA(get(checkbox.(fnames{i}),'UserData'),get(checkbox.(fnames{i}),'value'));
  end
  close(get(src,'parent'));

  %************************************************************%
function cancel_cbk(src,eventdata)
  close(get(src,'parent'));

