function gui_expctrl_noise(src,eventdata)
% Callback function for Experimental Control -> Model Noise menu item

% TODO % Timing noise for auditory/somatosensory error calculation and
% feedforward motor cmd

  h=figure('name','Experimental Control: Noise Specification',...
    'numbertitle','off','units','pixels','position',[100 100 640 480],...
    'menu','none','doublebuffer','on','color',[0.5 0.5 0.5]);
    
  % Split figure/frame into thirds: Acoustic Error noise, Somato Error
  % noise and FFcmd Amplitude noise
  %
  
  
  panel.border = [0.05 0.05]; % side and top borders
  panel.width = (1-4*panel.border(1))/3;
  panel.height = (1-2*panel.border(2));
  
  frames = {'AUDnoise','SOMnoise','FFnoise'};
  motorVar = {'ErrorNoise_sound','ErrorNoise_somatosensory','ErrorNoise_ffAmplitude'};
  frameTitle = {'Auditory Error','Somatosensory Error','Feedforward Command'};
  
  for n=1:3, % 3 sections
    distPar = DIVA('MotorCortex',motorVar{n});
    setappdata(h,frames{n},distPar);
    
    pos(1) = n*panel.border(1)+(n-1)*panel.width;
    pos(2) = panel.border(2)+2*panel.border(2);
    pos(3) = panel.width;
    pos(4) = panel.height-2*panel.border(2);
    
    panel.(frames{n}).main = uicontrol('style','frame','units','norm',...
      'position',[pos(1)+0.01*pos(3) pos(2)+0.825*pos(4) 0.98*pos(3) 0.15*pos(4)],...
      'backgroundcolor',[0.7 0.7 0.7]);
    panel.(frames{n}).title = uicontrol('style','text','parent',h,...
      'string',frameTitle{n},'FontUnits','pixels','FontSize',16,...
      'FontWeight','bold','Units','norm','position',...
      [pos(1)+0.05*pos(3) pos(2)+0.85*pos(4) 0.9*pos(3) 0.1*pos(4)],...
      'backgroundcolor',[0.7 0.7 0.7]);
    panel.(frames{n}).axes = axes('units','norm','parent',h, ...
      'position',[pos(1)+0.05*pos(3) pos(2)+0.25*pos(4) 0.9*pos(3) 0.5*pos(4)]);      
    %%% just plot N(0,1/4) for now
    if(distPar(2)==0)
      panel.(frames{n}).noiseDist = line(-1:0.001:1,((-1:0.001:1)==distPar(2))*3,...
        'tag',frames{n});
    else
      panel.(frames{n}).noiseDist = line(-1:0.001:1,normpdf(-1:0.001:1,distPar(1),distPar(2)),...
        'tag',frames{n});
    end
    axis off
      
    line([0 0],[-0.1 normpdf(0,0,1/10)],'color','k','linestyle','--');
    line([-1 -1],[-0.1 normpdf(0,0,1/10)],'color','k','linestyle',':');
    line([1 1],[-0.1 normpdf(0,0,1/10)],'color','k','linestyle',':');
    set(gca,'xlim',[-1 1],'ylim',[-0.1 3]);
    
    panel.(frames{n}).slideframe = uicontrol('style','frame','units','norm',...
      'position',[pos(1) pos(2) pos(3) 0.2*pos(4)],...
      'backgroundcolor',[0.7 0.7 0.7]);
    
    panel.(frames{n}).sliderBias = uicontrol('style','slider','units','norm',...
      'position',[pos(1)+0.05*pos(3) pos(2)+(0.2/3)*pos(4)+0.05*pos(4) ...
                  0.9*pos(3) 0.05*pos(4)],'min',-1,'max',1,'sliderstep',[0.01,0.1],...
                  'tag','bias','value',distPar(1),...%'tooltipstring',num2str(distPar(1)),...
                  'callback',{@slider_cbk,panel.(frames{n}).noiseDist});
    panel.(frames{n}).sliderVar = uicontrol('style','slider','units','norm',...
      'position',[pos(1)+0.05*pos(3) pos(2)+(0.1/3)*pos(4) ...
                  0.9*pos(3) 0.05*pos(4)],'min',0,'max',2,'sliderstep',[0.01,0.1],...
                  'tag','var','value',distPar(2),...
                  'callback',{@slider_cbk,panel.(frames{n}).noiseDist});
                
  % labels for sliders
    if(n==1)
      panel.labelBias = uicontrol('style','text','units','norm',...
        'position',[0 pos(2)+(0.2/3)*pos(4)+0.05*pos(4) ...
                    pos(1) 0.05*pos(4)],...
        'string','Bias','parent',h,'backgroundcolor',[0.5 0.5 0.5],...
        'FontSize',10);
      panel.labelVar = uicontrol('style','text','units','norm',...
        'position',[0 pos(2)+(0.1/3)*pos(4) pos(1) 0.05*pos(4)],...
        'string','Var','parent',h,'backgroundcolor',[0.5 0.5 0.5],...
        'FontSize',10);
    end
  end
  
  
  %----------------------------Set/cancel-------------------------------------%

  btn.ok=uicontrol('parent',h,'style','pushbutton','units','norm',...
    'position',[.8 .05 .05 .05],'FontUnits','norm',...
    'FontSize',.45,'String','OK',...
    'callback',{@ok_cbk,frames,motorVar});
  btn.cancel=uicontrol('parent',h,'style','pushbutton','units','norm',...
    'position',[.85 .05 .1 .05],'FontUnits','norm',...
    'FontSize',.45,'String','Cancel',...
    'callback',@cancel_cbk);

  %************************************************************%
  function ok_cbk(src,eventdata,frames,motorVar)    
    for n=1:length(frames),      
      distPar = getappdata(gcf,frames{n});
      DIVA('MotorCortex',motorVar{n},distPar);
    end      
    close(get(src,'parent'));
  
  %************************************************************%
  function cancel_cbk(src,eventdata)
    close(get(src,'parent'));

  %************************************************************%
  function slider_cbk(src,eventdata,handles)
    distPar = getappdata(gcf,get(handles,'tag'));
    switch(get(src,'tag'))
      case 'bias',
        distPar(1)=get(src,'value');
        %set(src,'tooltipstring',num2str(distPar(1)));
      case 'var',
        distPar(2)=get(src,'value');
    end
    if(distPar(2)==0)
      set(handles,'visible','on','ydata',(fix(1000*(-1:0.001:1))==fix(1000*distPar(1)))*3);
    else
      set(handles,'visible','on','ydata',normpdf(-1:0.001:1,distPar(1),distPar(2)));
    end
    setappdata(gcf,get(handles,'tag'),distPar);