function synthTarget(varargin)
  fmt = [];
  dur = [];
  out = [];
  if(~isempty(varargin))
    for n=1:size(varargin,1)
      fmt = [fmt, varargin(n,1:3)];
      dur = [dur, varargin(n,4)];
    end
  end

  % select from list, separate gui
  mainfig = figure('name','Synthesize Auditory Target','numbertitle','off',...
                   'units','norm','position',[.3 .3 .4 .4],'menubar','none');

  [pb.type pb.num pb.vnum pb.vowel pb.f0 pb.f1 pb.f2 pb.f3]= ...
      textread('data/verified_pb.data','%d%d%d%s%f%f%f%f');

  vowel_ind=[];
  for j=1:length(pb.type)
    if(pb.type(j)==1) % Male only
      if isempty(findstr(char(pb.vowel(j)),'*'))
        if isfield(vowel_ind,char(pb.vowel(j)))
          vowel_ind.(char(pb.vowel(j)))(end+1)=j;
        else
          vowel_ind.(char(pb.vowel(j)))(1)=j;
        end
      end
    end
  end
  names=fieldnames(vowel_ind);
  for n=1:length(names)
    meanVowel(n,:)=[mean(pb.f1(vowel_ind.(names{n}))),...
                    mean(pb.f2(vowel_ind.(names{n}))),...
                    mean(pb.f3(vowel_ind.(names{n})))];
  end
  example.iy='sweet';
  example.ih='Bill';
  example.ey='ate';
  example.eh='head';
  example.ae='after';
  example.er='bird';
  example.axr='creature';
  example.uh='butter';
  example.ah='about';
  example.ay='kite';
  example.aw='flower';
  example.aa='hot';
  example.ow='goat';
  example.oy='toy';
  example.ao='bought';
  example.uw='smooth';
  example.uh='wood';
  % /iy/ sweet, /ih/ bill, /ey/ ate, /eh/ head, /ae/ after
  % /er/ bird, /axr/ creature, /uh/ butter, /ax/ about
  % /ay/ kite, /aw/ flower, /aa/ hot, /ow/ goat,
  % /oy/ toy, /ao/ bought, /uw/ smooth, /uh/ wood

  uicontrol('style','text','parent',mainfig,...
            'units','norm','fontunits','norm',...
            'position',[0.1 0.86 0.1 0.05],'fontsize',0.7,...
            'HorizontalAlignment','center','string','Vowel');
  uicontrol('style','text','parent',mainfig,...
            'units','norm','fontunits','norm',...
            'position',[0.21 0.86 0.11 0.05],'fontsize',0.7,...
            'HorizontalAlignment','center','string','Example');

  vowel.label = uicontrol('style','text','tag','vowel','parent',mainfig,...
                          'units','norm','fontunits','norm',...
                          'position',[0.22 0.79 0.1 0.05],...
                          'backgroundcolor',[0.7 0.7 0.7],...
                          'fontsize',0.8,'string',example.(lower(names{1})));
  vowel.menu = uicontrol('style','popupmenu','tag','vowel','parent',mainfig,...
                         'string',names,'units','norm','fontunits','norm',...
                         'position',[0.1 0.8 0.1 0.05],'fontsize',0.8,...
                         'callback',{@setLabel,vowel.label,example,names});

  uicontrol('style','text','parent',mainfig,'units','norm','fontunits','norm',...
            'position',[0.5 0.86 0.3 0.05],'fontsize',0.7,'string','target',...
            'HorizontalAlignment','center');
  target.label = uicontrol('style','edit','tag','target','parent',mainfig,...
                           'units','norm','fontunits','norm',...
                           'position',[0.5 0.8 0.3 0.05],...
                           'fontsize',0.7,'horizontalalignment','center');

  target.guiaxes = axes('tag','target','parent',mainfig,...
                        'units','norm','yticklabel',[],...
                        'position',[0.1 0.2 0.8 0.5]);
  
  if(~isempty(fmt) & ~isempty(dur))
  else
    co=get(target.guiaxes,'colororder');
    for n=1:3,
      target.trace(n)=line('visible','off','color',co(n,:));
    end
  end
  target.sv_btn = uicontrol('style','pushbutton','tag','save',...
                            'units','norm','fontunits','norm',...
                            'position',[0.4 0.05 0.1 0.05],...
                            'fontsize',0.7,'string','Save',...
                            'parent',mainfig,...
                            'callback',{@saveTarget,target});
  target.cancel_btn = uicontrol('style','pushbutton','tag','save',...
                                'units','norm','fontunits','norm',...
                                'position',[0.5 0.05 0.1 0.05],...
                                'fontsize',0.7,'string','Cancel',...
                                'parent',mainfig,...
                                'callback','close');

  set(vowel.menu,'KeyPressFcn',{@doKeyPress,target,names,meanVowel});
  set(target.label,'KeyPressFcn',{@doKeyPress,target});

function setLabel(src,eventdata,vlabel,example,names)
  set(vlabel,'string',example.(lower(names{get(src,'value')})));

function doKeyPress(src,eventdata,target,names,meanVowel)
  switch(eventdata.Key)
   case 'return',
    set(target.trace,'visible','on');
    idx=get(src,'Value');
    set(target.label,'string',strcat(get(target.label,'string'),lower(names{idx})));
    ud=get(target.guiaxes,'UserData');
    ud=[ud;meanVowel(idx,:), 150];
    set(target.guiaxes,'UserData',ud);
    for n=1:3,
      xdata = [];
      ydata = [];
      tydata = [];
      for m=1:size(ud,1)
        xdata = [xdata, (m-1)*ud(m,4):(m-1)*ud(m,4)+ud(m,4)-1];
        if(m>1)
          tydata = [tydata;
                   (m-1)*ud(m-1,4)+round(ud(m,4)*0.25), ud(m,n);
                   (m-1)*ud(m-1,4)+round(ud(m,4)*0.75), ud(m,n)];
        else
          tydata = [tydata;
                   0, ud(m,n);
                   round(ud(m,4)*0.25), ud(m,n);
                   round(ud(m,4)*0.75), ud(m,n)];
        end
      end
      ydata = pchip(tydata(:,1),tydata(:,2),xdata);
      set(target.trace(n),'xdata',xdata,...
                        'ydata',ydata);
    end
    
   case 'backspace'
    lbl=get(target.label,'string');
    lbl(end-1:end)=[];
    set(target.label,'string',lbl);
    ud=get(target.guiaxes,'UserData');
    ud(end,:)=[];
    set(target.guiaxes,'UserData',ud);
    for n=1:3,
      xval=get(target.trace(n),'xdata');
      yval=get(target.trace(n),'ydata');
      if(length(xval)==1)
        break;
      else
        length(xval)
        if(length(xval>150))
          xval(end-150+1:end)=[];
          yval(end-150+1:end)=[];
        else
          xval=0;
          yval=0;
        end
        set(target.trace(n),'xdata',xval,'ydata',yval);
      end
    end
  end

function saveTarget(src,eventdata,target)
  xdata = [];
  ydata = [];
  for n=1:length(target.trace)
    xdata = [xdata; get(target.trace(n),'xdata')];
    ydata = [ydata; get(target.trace(n),'ydata')];
  end
  
  name = get(target.label,'string');
  dur = size(xdata,2);
  f0=100;
  agp=0.2;
  ag0=0;
  F0 = [(0:dur-1)' ...
        pchip([0 ceil(dur/3) dur],[f0-5 f0 f0-.25*f0],0:dur-1)' ...
        ones(dur,1)];
  AgP=[(0:dur-1)',repmat(agp,dur,1),ones(ceil(dur),1)];
  Ag0=[(0:dur-1)',repmat(ag0,dur,1),ones(ceil(dur),1)];
  f.mean=round(ydata)';
  f.var = repmat([400 2500 10000],dur,1);
  fileTarget.pathname=['Sessiontargets',filesep,'Session_',DIVA('SessionName'),filesep];
  fileTarget.filename=[name,'.mat'];
  if(~exist(fileTarget.pathname)),
    mkdir(fileTarget.pathname);
  end
  save([fileTarget.pathname,fileTarget.filename],'f','Ag0','AgP','F0','dur','name');
  DIVA('SoundChannel','sound',f.mean');
  DIVA('AuditoryCortexCategorical','label',name);
  DIVA('simulate');
  DIVA('GUI','ACC_AuditoryTargets',...
       DIVA('AuditoryCortexCategorical','AuditoryTargets'));
  DIVA('GUI','auTarget',fileTarget);
  DIVA('GUI','train_sound');
  close(get(src,'parent'));
