function [out]=calculateInverse(motor,mode,mode_error,mode_dim)  
  N=size(mode_error,2);
  InverseParameters=DIVA('MotorCortex','InverseParameters');
  
  for ninput=1:N,                  
    % use grid* notation for linearization of
    % articulatory/modal trajectories.  Form Jacobian from
    % linear forms and perform inverse operation on Jacobian
    
    nMotor = size(motor,1);
    nMode  = size(mode,1);
    
    current_motor = motor(:,ninput);
    current_mode  = mode(:,ninput);
    
    grid_motor = current_motor*ones(1,nMotor)+...
        InverseParameters.DELTA*diag(ones(1,nMotor));
    grid_mode   = current_mode(:,ones(1,nMotor));

    % Determine the Jacobian
    if(nMode == DIVA('AuditoryCortex','nFormants'))
      temp = DIVA('VocalTract','motor',grid_motor);
      xMode=temp{1};      
    elseif(nMode == DIVA('Sensory','nSensory'))
      temp = DIVA('VocalTract','motor',grid_motor);
      xMode = DIVA('Sensory','configuration',temp{2});
    end
    
    J1 = (xMode-grid_mode)./InverseParameters.DELTA;
    
    grid_motor = current_motor*ones(1,nMotor)-...
        InverseParameters.DELTA*diag(ones(1,nMotor));
    grid_mode  = current_mode(:,ones(1,nMotor));
    
    if(nMode == DIVA('AuditoryCortex','nFormants'))
      temp = DIVA('VocalTract','motor',grid_motor);
      xMode=temp{1};      
    elseif(nMode == DIVA('Sensory','nSensory'))
      temp = DIVA('VocalTract','motor',grid_motor);
      xMode = DIVA('Sensory','configuration',temp{2});
    end    

    J2 = (xMode-grid_mode)./-InverseParameters.DELTA;
    
    J = 0.5*(J1+J2);
    
    % Determine the inverse of the Jacobian
    %   Various tricks can be played here related to regularization
    %   This step is crucial for the inverse transformation and various
    %   different techniques can be applied to create a better solution
    %   At present the straightforward pseudoinverse is applied.
    
    J(find(isnan(J(:)))) = 0;
    [U,S,V]=svd(J,0);
    
    Jinv = pinv(J'*J+InverseParameters.LAMBDA*...
                eye(size(J,2)))*J';
    
    Jnull=V(:,nMode+1:nMotor); 
    Jnull=Jnull*pinv(Jnull'*Jnull)*Jnull';    
    
%     out=InverseParameters.ALPHA*...
%         Jinv*mode_error - InverseParams.BETA*...
%         Jnull*repmat(motor(:,ninput),1,N);

out(:,ninput) = InverseParameters.ALPHA*Jinv*mode_error(:,ninput) -...
    InverseParameters.BETA*Jnull*motor(:,ninput);

%     for ntarget = 1:N,
%       % Jacobian formula
%       out(:,ntarget) = InverseParameters.ALPHA*...
%           Jinv*mode_error(:,ntarget) - InverseParameters.BETA*...
%           Jnull*motor(:,ninput); % was Jnull*motor -> presumes one input/output
%      end
  end 
