function [y,idx] = equalstep(x,step,type);
% EQUALSTEP Resamples to equal step-size
%
% [y,idx]=equalstep(x,stepsize) resamples the vector 'x' 
% to a new vector y=x(idx) (with linear interpolation
% for non-integer idx values) so that abs(diff(y)) is 
% constant and equal to 'stepsize'.
%
% If x is a matrix it considers each column of x as a data 
% point: It attempts sqrt(sum(abs(diff(y,1,2)).^2,1)) = stepsize
% where y=x(:,idx)).
%
% [y,idx]=equalstep(x,n,'fixedlength') resamples so that y has
% n elements (with equal stepsize)
%

if sum(size(x)>1)==1, x=x(:).'; end
dx=1e-5+sqrt(sum(abs(diff(x,1,2)).^2,1)); dx(isnan(dx))=1e-5;
z=cumsum([0,dx]);
n=size(x,2);

if nargin>2 & strcmp(lower(type),'fixedlength'), step=z(end)/(step-1); end
idx=interp1(z,1:n,0:step:z(end));
y=interp1(1:n,x.',idx.').';
