function [Y,idx_X]=Sample_Window(X, Wlength, Nlength, Type, Extent);

% SAMPLE_FUNCTION Signal windowing
% Y=Sample_Window(X, WINDOW, NOVERLAP, Type, Extent);
%     X        :  Column vector (or N-dimensional array of column vectors)
%     WINDOW   :  # samples of window
%     NOVERLAP :  # overlapping samples
%     Type     :  Windowing function (['none'],'hamming','hanning','boxcar')
%     Extent   :  Border options (['valid'],'same')
% returns matrix Y (or N+1 dimensional array) resulting from splitting
% the signal X into overlapping windowed segments.
% 

% alfnie@bu.edu
% ?/99


sX=[size(X),1];
if nargin<2 | isempty(Wlength), Wlength=sX(1); end 
if nargin<3 | isempty(Nlength), Nlength=floor(Wlength/2); end
if nargin<4 | isempty(Type), Type='none'; end
if nargin<5 | isempty(Extent), Extent='tight'; end
if lower(Type(1))=='c', docentering=1; Type=Type(2:end); else, docentering=0; end

X=[zeros([Nlength,sX(2:end)]); X; zeros([Nlength,sX(2:end)])];
sY=1+floor((size(X,1)-Wlength)/(Wlength-Nlength));

idx_W=repmat((1:Wlength)',[1,sY]); 
idx_X=idx_W + ones(Wlength,1)*(Wlength-Nlength)*(0:sY-1);

switch(lower(Type)),
case 'hamming',
    W=hamming(Wlength);
case 'hanning',
    W=hanning(Wlength);
case 'boxcar',
    W=boxcar(Wlength);
case 'none',
    W=ones(Wlength,1);
end

if docentering,
    e=convn(X.^2,hamming(Wlength-Nlength+1),'same');
    E=e(idx_X(:),:).*W(idx_W(:),ones(1,prod(sX(2:end))));
    E=reshape(E,[Wlength,sY,sX(2:end)]);
    [nill,idxE]=max(E,[],1);
    idx_X=max(1,min(size(X,1),idx_X+repmat(idxE-floor(Wlength/2),[Wlength,1])));
end

if strcmp(lower(Type),'none'), Y=X(idx_X(:),:); else, Y=X(idx_X(:),:).*W(idx_W(:),ones(1,prod(sX(2:end)))); end
Y=reshape(Y,[Wlength,sY,sX(2:end)]);

idx_X=idx_X-Nlength*sX(2);
idx_X(idx_X<=0 | idx_X>prod(sX))=nan;

switch(lower(Extent)),
case {'valid','tight'},
    idx=all(~isnan(idx_X),1);
    Y=reshape(Y(:,idx,:),[Wlength,sum(idx),sX(2:end)]);
    idx_X=idx_X(:,idx);
case 'same',
    idx=~isnan(idx_X(floor(end/2),:));
    Y=reshape(Y(:,idx,:),[Wlength,sum(idx),sX(2:end)]);
    idx_X=idx_X(:,idx);
end
